{
' This file is a skeleton file that can be used to add XMM support for a
' completely new Propeller platform, or for a memory add-on board to be
' used with multiple Propeller platforms.
'
' The first step is to decide on the Platform or Board name, and the Catalina 
' symbol that will be used to specify it on the command line (using the -C 
' command line option). In this file We will assume the platform is to be 
' called "New_Board", and we will use the symbol NEW_BOARD. 
'
' 1. The first step is to rename this file to be "New_Board_XMM.inc".
' 
' 2. The next step is to update XMM.inc. There are two options - select (a)
'    for new platforms with XMM hardware "built-in", or (b) for new "add-on" 
'    XMM boards (but not both!):
'
' a. If this represents a completely new platform, add the following lines 
'    to the file END of the list in XMM.inc, just before the line that says
'    "#elseifdef CUSTOM":
'
'      #elseifdef NEW_BOARD
'      #include "New_Board_XMM.inc"
'
'    Note that for new platforms, you will also need to update CFG.inc, DEF.inc
'    and HMI.inc in a similar manner.
'
' b. If this is an add-on board that can be used on other platforms, add the
'    above lines to the START of the list in XMM.inc, just above the first
'    platform - i.e. just before the line that says "#elseifdef HYDRA":
'
'      #elseifdef NEW_BOARD
'      #include "New_Board_XMM.inc"
'
' 3. The final step is to add the required code to the API functions below. 
'
'    Note that if you only intend to use this board in conjunction with the 
'    Cache, you only need to implement the "MINIMAL API FUNCTIONS". 
'
'    If you want to use this board WITHOUT the cache, you also need to 
'    implement the "DIRECT API FUNCTIONS". Also, delete the CACHE CHECK
'    code below.
'
'    If your board implelements FLASH RAM, you will also need to implement
'    the "FLASH API FUNCTIONS". Also, delete the FLASH CHECK code below.
}
'===============================================================================
{
' The XMM functions included are controlled via Catalina symbols defined 
' internally as required within various Catalina files:
'
'   CACHED             : defined if CACHE in use (any size)
'
'   NEED_FLASH         : defined if FLASH support required
'
'   NEED_XMM_READLONG  : defined if XMM_ReadLong (and XMM_ReadMult) required
'
'   NEED_XMM_WRITELONG : defined if XMM_WriteLong (and XMM_WriteMult) required
'
'   NEED_XMM_READPAGE  : defined if XMM_ReadPage required
'
'   NEED_XMM_WRITEPAGE : defined if XMM_WritePage required
}
'=============================== CACHE CHECK ===================================
'
' If this platform does not need the cache enabled, delete the following:
'
'#ifndef CACHED
'#error : PLATFORM REQUIRES CACHE OPTION (CACHED_1K .. CACHED_8K or CACHED)
'#endif
'
'=============================== FLASH CHECK ===================================
'
' If this platform has FLASH RAM, delete the following:
'
#ifdef NEED_FLASH
#error : FLASH NOT SUPPORTED ON THIS PLATFORM
#endif
'
'=========================== MINIMAL API FUNCTIONS =============================

CON

#include "CUSTOM_XMM_DEF.inc"
'
DAT
'
'-----------------------------------Activate SRAM Chip------------------------------------------ 
XMM_Activate          or     dira,MemClkHigh             'Set MemClkPin As Output
                      and    outa,MemClkLow              'Force MemClkLow Initially
                      or     dira,RamCsHigh              'Set RamCsPin As Output
                      or     outa,RamCsHigh              'Set RamCsHigh (Disable Ram)
                      or     dira,RomCsHigh              'Set RomCsPin As Output
                      or     outa,RomCsHigh              'Set RomCsHigh (Disable Rom)
                      call   #MakeRamQuadMode            'Force Ram Into Quad Mode
                      call   #MakeRamSeq                 'Set SRAM For Seq Read/Write
XMM_Activate_ret      ret
'
'---------------------------------TriState Memory------------------------------------------
XMM_Tristate          and    dira,QuadBusIn              'TriState Quad Bus
                      and    dira,MemClkLow              'TriState MemClk
                      and    dira,RamCsLow               'TriState RamCs
                      and    dira,RomCsLow               'TriState RomCs
XMM_Tristate_ret      ret
'
#ifdef NEED_XMM_WRITEPAGE
'-----------------------------------XMM_WritePage (HubRam -> XMM)--------------------------
XMM_WritePage         and    outa,RamCsLow               'Set RamCsLow (SRAM Active)
                      mov    MemAddr,XMM_Addr            'Copy XMM_Addr
                      shl    MemAddr,#08                 'Remove Upper Byte
                      shr    MemAddr,#08                 'Reposition MemAddr
                      mov    MemCmd,#WDR                 'Write Mode
                      call   #SendMemCmd                 'Send Write Command
:WritePageLoop        rdbyte MemTxD,Hub_Addr             'Get Byte From HubRam
                      mov    NibsUsed,#02                'Send 2 Nibbles (1 Byte)
                      call   #SendQuadData               'Write The Data
                      add    Hub_Addr,#01                'Increment HubRam Address
                      add    XMM_Addr,#01                'Increment XMM_Addr
                      djnz   XMM_Len,#:WritePageLoop     'Repeat Until Done
                      or     outa,RamCsHigh              'Set RamCsHigh (SRAM Inactive)
XMM_WritePage_ret     ret
#endif
'
#ifdef NEED_XMM_READPAGE
'----------------------------------- XMM_ReadPage (XMM -> HubRam)--------------------------
XMM_ReadPage          and    outa,RamCsLow               'Set RamCsLow (SRAM Active)
                      mov    MemAddr,XMM_Addr            'Copy XMM_Addr
                      shl    MemAddr,#08                 'Remove Upper Byte
                      shr    MemAddr,#08                 'Reposition MemAddr
                      mov    MemCmd,#RDR                 'Read Mode
                      call   #SendMemCmd                 'Send Read Command
:ReadPageLoop         xor    MemRxD,MemRxD               'Clear The Read Byte Register
                      mov    NibsUsed,#02                'Retrieve 2 Nibbles (1 Byte)
                      call   #GetQuadData                'Get The Data
                      wrbyte MemRxD,Hub_Addr             'Write The Byte To HubRam
                      add    Hub_Addr,#01                'Increment HubRam Address
                      add    XMM_Addr,#01                'Increment XMM_Addr
                      djnz   XMM_Len,#:ReadPageLoop      'Repeat Until Done
                      or     outa,RamCsHigh              'Set RamCsHigh (SRAM Inactive)
XMM_ReadPage_ret      ret
#endif
'
'============================ DIRECT API FUNCTIONS =============================
'
DAT
'
#ifdef NEED_XMM_READLONG 
'---------------------------------XMM ReadLong (XMM -> Cog)---------------------------------
XMM_ReadLong          and    outa,RamCsLow               'Set RamCsLow (SRAM Active)
                      mov    MemAddr,XMM_Addr            'Get XMM_Addr
                      shl    MemAddr,#08                 'Remove Upper Byte
                      shr    MemAddr,#08                 'Restore MemAddr
                      mov    MemCmd,#RDR                 'Read Mode
                      call   #SendMemCmd                 'Send Read Command
                      xor    MemRxD,MemRxD               'Clear Read Register
                      mov    NibsUsed,#08                'Get 8 Nibbles,4 Bytes,2 Words,1 Long
                      call   #GetQuadData                'Get The Data
                      mov    XMM_Temp,MemRxD             'Save In XMM_Dst Register
                      add    XMM_Addr,#04                'Long Read -> Increment By 4
                      or     outa,RamCsHigh              'Set RamCsHigh (SRAM Inactive)
XMM_ReadLong_ret      ret

'---------------------------------XMM ReadMult (XMM -> Cog)---------------------------------
XMM_ReadMult          and    outa,RamCsLow               'Set RamCsLow (SRAM Active)
                      mov    MemAddr,XMM_Addr            'Get XMM_Addr
                      shl    MemAddr,#08                 'Remove Upper Byte
                      shr    MemAddr,#08                 'Restore MemAddr
                      mov    MemCmd,#RDR                 'Read Mode
                      call   #SendMemCmd                 'Send Read Command
                      shl    XMM_Len,#01                 'Bytes -> Nibbles
                      xor    MemRxD,MemRxD               'Clear Read Register
:ReadMultLoop         mov    NibsUsed,#02                'Read 2 Nibbles,1 Byte
                      call   #GetQuadData                'Get The Byte
                      add    XMM_Addr,#01                'Increment
                      djnz   XMM_Len,:ReadMultLoop       'Loop Until Done
                      mov    XMM_Temp,MemRxD             'Save In XMM_Dst Register
                      or     outa,RamCsHigh              'Set RamCsHigh  (SRAM Inactive)
XMM_ReadMult_ret      ret
#endif          
'
#ifdef NEED_XMM_WRITELONG
'---------------------------------XMM_WriteLong (Cog -> XMM)-------------------------------
XMM_WriteLong         and    outa,RamCsLow               'Set RamCsLow (SRAM Active)
                      mov    MemAddr,XMM_Addr            'Get XMM_Addr
                      shl    MemAddr,#08                 'Remove Upper Byte
                      shr    MemAddr,#08                 'Restore MemAddr
                      mov    MemCmd,#WDR                 'Write Mode
                      call   #SendMemCmd                 'Send Write Command
                      mov    MemTxD,XMM_Temp             'Get Data From XMM_Src
                      mov    NibsUsed,#08                'Send 8 Nibbles,4 Bytes,2 Words,1 Long
                      call   #SendQuadData               'Send The Data
                      add    XMM_Addr,#04                'Long Write -> Increment By 4
                      or     outa,RamCsHigh              'Set RamCsHigh (SRAM Inactive)
XMM_WriteLong_ret     ret

'---------------------------------XMM ReadMult (XMM -> Cog)---------------------------------
XMM_ReadMult          and    outa,RamCsLow               'Set RamCsLow (SRAM Active)
                      mov    MemAddr,XMM_Addr            'Get XMM_Addr
                      shl    MemAddr,#08                 'Remove Upper Byte
                      shr    MemAddr,#08                 'Restore MemAddr
                      mov    MemCmd,#RDR                 'Read Mode
                      call   #SendMemCmd                 'Send Read Command
                      shl    XMM_Len,#01                 'Bytes -> Nibbles
                      xor    MemRxD,MemRxD               'Clear Read Register
:ReadMultLoop         mov    NibsUsed,#02                'Read 2 Nibbles,1 Byte
                      call   #GetQuadData                'Get The Byte
                      add    XMM_Addr,#01                'Increment
                      djnz   XMM_Len,:ReadMultLoop       'Loop Until Done
                      mov    XMM_Temp,MemRxD              'Save In XMM_Dst Register
                      or     outa,RamCsHigh              'Set RamCsHigh  (SRAM Inactive)
XMM_ReadMult_ret      ret
#endif
'
'============================= FLASH API FUNCTIONS =============================
'
DAT
'
#ifdef NEED_FLASH
'
' XMM_FlashActivate : Activate FLASH bus (equivalent to XMM_Activate for FLASH).
' On entry:
'    None.
' On exit:
'    None.
'
XMM_FlashActivate
              nop                               ' <== INSERT CODE HERE
XMM_FlashActivate_ret
              ret
'            
'XMM_FlashTristate : Give up the FLASH bus. Use this to use other hardware
'                    that shares pins with the FLASH RAM. To reactivate the 
'                    FLASH bus, call XMM_FlashActivate.
' On entry:
'    None.
' On exit:
'    None.
'            
XMM_FlashTristate
              nop                               ' <== INSERT CODE HERE
XMM_FlashTristate_ret
              ret
'
' XMM_FlashWritePage : Write (program) bytes from Hub RAM to FLASH
' On entry:
'    XMM_Addr  (32-bit): destination address in FLASH
'    Hub_Addr  (32-bit): source address in Hub
'    XMM_Len   (32-bit): number of bytes to write 
' On exit:
'    XMM_Addr incremented by the number of bytes written
'
XMM_FlashWritePage
              nop                               ' <== INSERT CODE HERE
XMM_FlashWritePage_ret
              ret
'
' XMM_FlashReadPage : Read bytes from FLASH to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in FLASH
'    Hub_Addr  (32-bit): destination address in Hub
'    XMM_Len   (32-bit): number of bytes to read 
' On exit:
'    XMM_Addr incremented by the number of bytes read
'
XMM_FlashReadPage
              nop                               ' <== INSERT CODE HERE
XMM_FlashReadPage_ret
              ret
'
' XMM_FlashCheckEmpty : Check bytes from FLASH are all $FF.
' On Entry: 
'    XMM_Addr  (32-bit): source address in Flash
'    XMM_Len   (32-bit): number of bytes to read 
' On Exit:
'    Z flag set if all empty, Z flag reset on fail (and failed byte in outx)
'
XMM_FlashCheckEmpty
              nop                               ' <== INSERT CODE HERE
XMM_FlashCheckEmpty_ret
              ret
'
' XMM_FlashComparePage : Compare bytes from FLASH with Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in Flash
'    Hub_Addr  (32-bit): destination address in main memory
'    XMM_Len   (32-bit): number of bytes to read 
' On Exit:
'    Z flag set if all the same, Z flag reset on fail (and failed byte in outx)
'
XMM_FlashComparePage
              nop                               ' <== INSERT CODE HERE
XMM_FlashComparePage_ret
              ret
'
' XMM_FlashEraseChip : Erase the whole FLASH chip
' On entry:
'    None.
' On exit:
'    None.
'
XMM_FlashEraseChip
              nop                               ' <== INSERT CODE HERE
XMM_FlashEraseChip_ret
              ret
'
' XMM_FlashEraseBlock : Erase a 4k block of the FLASH chip
' On entry:
'    XMM_Addr  (32-bit): address in FLASH of 4K block to erase.
' On exit:
'    None.
'
XMM_FlashEraseBlock
              nop                               ' <== INSERT CODE HERE
XMM_FlashEraseBlock_ret
              ret
'
' XMM_FlashUnprotect : Unprotect the entire FLASH chip
' On entry:
'    None.
' On exit:
'    None.
'
XMM_FlashUnprotect
              nop                               ' <== INSERT CODE HERE
XMM_FlashUnprotect_ret
              ret
'
' XMM_FlashWriteEnable : Enable Writing to the FLASH chip
' On entry:
'    None.
' On exit:
'    None.
'
XMM_FlashWriteEnable
              nop                               ' <== INSERT CODE HERE
XMM_FlashWriteEnable_ret
              ret
'
' XMM_FlashWaitUntilDone : Wait until previous FLASH request complete
' On entry:
'    None.
' On exit:
'    outx : contents of the FLASH status regiter.
'
XMM_FlashWaitUntilDone
              nop                               ' <== INSERT CODE HERE
XMM_FlashWaitUntilDone_ret
              ret
'
#endif
'
'=======================Required Functions For SRAM Read/Write=================================
'
'--------------------------------'Force Serial Memory Into Quad Mode---------------------------
MakeQuadMode          or     dira,SPIHi              'Set Ram_Data_D0 As Output
                      or     outa,SPIHi              'Set Initial Output
                      and    outa,SPILo              'Output = 0
                      call   #ClkStrobe              'Latch D7 = 0 \
                      call   #ClkStrobe              'Latch D6 = 0  \
                      or     outa,SPIHi              'Output = 1     Ox30 \
                      call   #ClkStrobe              'Latch D5 = 1  /      \
                      call   #ClkStrobe              'Latch D4 = 1 /        \_0x38
                      call   #ClkStrobe              'Latch D3 = 1 \        /
                      and    outa,SPILo              'Output = 0    \      /
                      call   #ClkStrobe              'Latch D2 = 0   0x08 /
                      call   #ClkStrobe              'Latch D1 = 0  /  
                      call   #ClkStrobe              'Latch D0 = 0 /
MakeQuadMode_ret      ret                            '0x38 Transmission Complete 
'
'----------------------------------Strobe The Memory Clock Line--------------------------------
ClkStrobe             or     outa,MemClkHigh         'Set MemClkHigh
                      and    outa,MemClkLow          'Set MemClkLow
ClkStrobe_ret         ret                                                         
'                                            
'--------------------------------Send 0x38 To SRAM To Force Quad Mode--------------------------  
MakeRamQuadMode       and    outa,RamCsLow           'Set RamCsLow  (SRAM Active)
                      call   #MakeQuadMode           'Call QuadMode 
                      or     outa,RamCsHigh          'Set RamCsHigh (SRAM Inactive)
MakeRamQuadMode_ret   ret 
'
'-------------------------------Make SRAM Use Sequential R/W Access----------------------------
MakeRamSeq            or     dira,QuadBusOut         'Set QuadBus As Output
                      and    outa,RamCsLow           'Set RamCsLow (Enable SRAM)
                      mov    MemTxD,#WMR             'Set Write Mode Register
                      shl    MemTxD,#08              'Move To Next Byte
                      or     MemTxD,#$40              'Select Seq Access
                      mov    NibsUsed,#04            'Send 4 Nibbles,2 bytes,1 Word
                      call   #SendQuadData           'Send The Data
                      or     outa,RamCsHigh          'Set RamCsHigh (SRAM Inactive)
MakeRamSeq_ret        ret
'
'-------------------------------------Output Data To XMM Memory--------------------------------
SendQuadData          mov    MemSlide,NibsUsed       'Load Nibbles To Send
                      sub    MemSlide,#01            'Subtract 1
                      shl    MemSlide,#02            'Multiply By 4 [MemSlide = 4 * (NibsUsed - 1)]
:SendNibble           and    outa,ClearQuadBus       'Clear QuadBus & MemClk
                      mov    MemTemp,MemTxD          'Load MemTxD
                      shr    MemTemp,MemSlide        'Move NibbleX -> Nibble0
                      and    MemTemp,#$0F            'Only Keep Nibble0
                      shl    MemTemp,PinJustify      'Pin Justify The Data
                      or     outa,MemTemp            'Write MemTemp To QuadBus
                      sub    MemSlide,#04            'Reposition To Next Nibble
                      or     outa,MemClkHigh         'Latch MemTemp To SRAM
                      djnz   NibsUsed,#:SendNibble   'Repeat For Next Nibble
                      and    outa,ClearQuadBus       'Clear QuadBus & MemClk
SendQuadData_ret      ret
'
'----------------------------------Read Data From XMM Memory-----------------------------------                      
GetQuadData           and    dira,QuadBusIn          'Set QuadBus For Input                      
:GetNibble            mov    MemTemp,ina             'Read Quad Bus
                      shr    MemTemp,PinJustify      'Pin Justify The Data
                      and    MemTemp,#$0F            'Keep Lowest Nibble
                      shl    MemRxD,#04              'Slide Nibble0 Left
                      or     MemRxD,MemTemp          'Load MemTxD With New Nibble
                      or     outa,MemClkHigh         'Set MemClkHigh (Ack Receipt Of Nibble)
                      and    outa,MemClkLow          'Set MemClkLow                      
                      djnz   NibsUsed,#:GetNibble    'Loop Until Done
GetQuadData_ret       ret                                            
'
'-------------------------Send Write Cmd And Address To Serial Memory---------------------------                     
SendMemCmd            or     dira,QuadBusOut         'Set QuadBus To Output
                      mov    MemTxD,MemCmd           'Load Cmd Byte
                      shl    MemTxD,#24              'Move To Upper Byte
                      or     MemTxD,MemAddr          'Include Address In Lower 3 Bytes
                      mov    NibsUsed,#08            '8 Nibbles (4 Bytes) To Send
                      call   #SendQuadData           'Output The Info
                      mov    MemTemp,MemCmd          'Copy MemCmd Into MemTemp
                      sub    MemTemp,#RDR            'Was This A Read Data Cmd (0x03)?
                      tjz    MemTemp,#:ReadMode      'If Yes, Goto ReadMode
                      mov    MemTemp,MemCmd          'Reload MemCmd
                      sub    MemTemp,#RMR            'Was This A Read Mode Cmd (0x05)?
                      tjz    MemTemp,#:ReadMode      'If Yes, Goto ReadMode
                      jmp    #SendMemCmd_ret         'Otherwise Return
:ReadMode             and    dira,QuadBusIn          'Set QuadBus For Input
                      or     outa,MemClkHigh         'Set MemClkHigh (1st Dummy Clock)
                      and    outa,MemClkLow          'Set MemClkLow
                      or     outa,MemClkHigh         'Set MemClkHigh (2nd Dummy Clock)
                      and    outa,MemClkLow          'Set MemClkLow
SendMemCmd_ret        ret

'===========================Required Variables For SRAM Read/Write=====================
     
RamCsHigh             long   Ram_Cs_High    
RamCsLow              long   Ram_Cs_Low     

RomCsHigh             long   Rom_Cs_High    
RomCsLow              long   Rom_Cs_Low    

MemClkHigh            long   Mem_Clk_High   
MemClkLow             long   Mem_Clk_Low    

SPIHi                 long   SPI_Hi             
SPILo                 long   SPI_Lo         

QuadBusOut            long   Quad_Bus_Out  
QuadBusIn             long   Quad_Bus_In    

ClearQuadBus          long   Clear_Quad_Bus     

PinJustify            long   Quad_Data_D0

MemAddr               long   0
MemTxD                long   0
MemRxD                long   0
MemCmd                long   0
NibsUsed              long   0 
MemTemp               long   0
MemSlide              long   0


'================================= XMM VARIABLES ===============================
'
DAT
'
outx          long    $0
XMM_Temp      long    $0
'
'===============================================================================
'
